@@ -5,42 +5,18 @@ no strict;
 @ISA = qw( Exporter Mail::Mbox::MessageParser );
 
 use strict;
-use warnings 'all';
+use Mail::Mbox::MessageParser;
+use Carp;
 
-our $VERSION = '1.00';
+use vars qw( $VERSION $DEBUG $GREP_DATA );
 
-our $DEBUG = 0;
+$VERSION = '1.5.0';
 
-our $GREP_DATA = {};
+$GREP_DATA = {};
 
-#-------------------------------------------------------------------------------
-
-sub dprint
-{
-  return Mail::Mbox::MessageParser::dprint @_;
-}
-
-#-------------------------------------------------------------------------------
-
-sub _HAS_GREP
-{
-  my $temp = `grep --help 2>&1`;
-
-  if ($temp =~ /usage/i && $temp =~ /extended-reg/i &&
-    $temp =~ /byte-offset/i && $temp =~ /line-numb/i)
-  {
-    return 1;
-  }
-  else
-  {
-    return 0;
-  }
-}
-
-#-------------------------------------------------------------------------------
-
-die "Can not load " . __PACKAGE__ . ": GNU grep is not installed.\n"
-  unless _HAS_GREP();
+*DEBUG = \$Mail::Mbox::MessageParser::DEBUG;
+*dprint = \&Mail::Mbox::MessageParser::dprint;
+sub dprint;
 
 #-------------------------------------------------------------------------------
 
@@ -52,30 +28,27 @@ sub new
   my $self  = {};
   bless ($self, $class);
 
-  die "Need file_name option" unless defined $options->{'file_name'};
-  die "Need file_handle option" unless defined $options->{'file_handle'};
+  carp "Need file_name option" unless defined $options->{'file_name'};
+  carp "Need file_handle option" unless defined $options->{'file_handle'};
+
+  return "GNU grep not installed"
+    unless defined $Mail::Mbox::MessageParser::PROGRAMS{'grep'};
 
   $self->{'file_handle'} = undef;
   $self->{'file_handle'} = $options->{'file_handle'}
     if exists $options->{'file_handle'};
 
   $self->{'file_name'} = $options->{'file_name'};
+  $self->{'file_name'} = $options->{'file_name'};
 
-  $self->{'end_of_file'} = 0;
-
-  # The line number of the last read email.
-  $self->{'email_line_number'} = 0;
-  # The offset of the last read email.
-  $self->{'email_offset'} = 0;
-  # The length of the last read email.
-  $self->{'email_length'} = 0;
-
-  $self->{'email_number'} = 0;
-
+  $self->reset();
 
   _READ_GREP_DATA($self->{'file_name'})
     unless defined $GREP_DATA->{$self->{'file_name'}};
 
+  return "Couldn't read grep data"
+    unless defined $GREP_DATA->{$self->{'file_name'}};
+
   return $self;
 }
 
@@ -85,7 +58,8 @@ sub reset
 {
   my $self = shift;
 
-  seek $self->{'file_handle'}, length($self->{'prologue'}), 0;
+  seek $self->{'file_handle'}, length($self->{'prologue'}), 0
+    if defined $self->{'file_handle'} && defined $self->{'prologue'};
 
   $self->{'end_of_file'} = 0;
 
@@ -101,7 +75,7 @@ sub _read_prologue
 {
   my $self = shift;
 
-  Mail::Mbox::MessageParser::dprint "Reading mailbox prologue using grep";
+  dprint "Reading mailbox prologue using grep";
 
   my $prologue_length = $GREP_DATA->{$self->{'file_name'}}{'offsets'}[0];
   my $bytes_read = 0;
@@ -121,8 +95,14 @@ sub _READ_GREP_DATA
 
   my @lines_and_offsets;
 
+  dprint "Reading grep data";
+
   {
-    my @grep_results = `grep --extended-regexp --line-number --byte-offset '^(X-Draft-From: .*|X-From-Line: .*|From [^:]+(:[0-9][0-9]){1,2}( +([A-Z]{2,3}|[+-]?[0-9]{4})){1,3}( remote from .*)?)\$' $filename`;
+    my @grep_results;
+    
+    @grep_results = `$Mail::Mbox::MessageParser::PROGRAMS{'grep'} --extended-regexp --line-number --byte-offset --binary-files=text "^(X-Draft-From: .*|X-From-Line: .*|From [^:]+(:[0-9][0-9]){1,2} ([A-Z]{2,3} [0-9]{4}|[0-9]{4} [+-][0-9]{4}|[0-9]{4})( remote from .*)?)\r?\$" "$filename"`;
+
+    dprint "Read " . scalar(@grep_results) . " lines of grep data";
 
     foreach my $match_result (@grep_results)
     {
@@ -136,8 +116,7 @@ sub _READ_GREP_DATA
   {
     if ($match_number == $#lines_and_offsets)
     {
-      my $filesize = -s $filename;
-      $GREP_DATA->{$filename}{'lengths'}[$match_number] =
+      my $filesize = -s $filename; $GREP_DATA->{$filename}{'lengths'}[$match_number] =
         $filesize - $lines_and_offsets[$match_number]{'byte offset'};
     }
     else
@@ -163,8 +142,6 @@ sub read_next_email
 {
   my $self = shift;
 
-  Mail::Mbox::MessageParser::dprint "Using grep data" if $DEBUG;
-
   $self->{'email_line_number'} =
     $GREP_DATA->{$self->{'file_name'}}{'line_numbers'}[$self->{'email_number'}];
   $self->{'email_offset'} =
@@ -193,9 +170,12 @@ sub read_next_email
     # Keep looking if the header we found is part of a "Begin Included
     # Message".
     my $end_of_string = substr($email, -200);
+    my $endline = $self->{'endline'};
     if ($end_of_string =~
-        /\n-----(?: Begin Included Message |Original Message)-----\n[^\n]*\n*$/i)
+        /$endline-----(?: Begin Included Message |Original Message)-----$endline[^\r\n]*(?:$endline)*$/i)
     {
+      dprint "Incorrect start of email found--adjusting grep data";
+
       $GREP_DATA->{$self->{'file_name'}}{'lengths'}[$self->{'email_number'}] +=
         $GREP_DATA->{$self->{'file_name'}}{'lengths'}[$self->{'email_number'}+1];
 
@@ -255,18 +235,6 @@ Mail::Mbox::MessageParser::Grep - A GNU grep-based mbox folder reader
 
   #!/usr/bin/perl
 
-  unless (eval 'require Mail::Mbox::MessageParser::Grep;')
-  {
-    if ($@ =~ /GNU grep is not installed/)
-    {
-      die "GNU grep is not installed\n";
-    }
-    else
-    {
-      die $@;
-    }
-  }
-
   use Mail::Mbox::MessageParser::Grep;
 
   my $filename = 'mail/saved-mail';
@@ -276,8 +244,10 @@ Mail::Mbox::MessageParser::Grep - A GNU grep-based mbox folder reader
     new Mail::Mbox::MessageParser::Grep( {
       'file_name' => $filename,
       'file_handle' => $filehandle,
+      'file_handle' => $filehandle,
     } );
 
+  die $folder_reader unless ref $folder_reader;
   
   # Any newlines or such before the start of the first email
   my $prologue = $folder_reader->prologue;
@@ -307,7 +277,7 @@ the Mail::Mbox::MessageParser documentation.
 =over 4
 
 =item $ref = new( { 'file_name' => <mailbox file name>,
-                    'file_handle' => <mailbox file handle> });
+                    'file_handle' => <mailbox file handle>, });
 
     <file_name> - The full filename of the mailbox
     <file_handle> - An opened file handle for the mailbox
@@ -316,6 +286,9 @@ The constructor for the class takes two parameters. I<file_name> is the
 filename of the mailbox.  The I<file_handle> argument is the opened file
 handle to the mailbox. Both arguments are required.
 
+Returns a reference to a Mail::Mbox::MessageParser object, or a string
+describing the error.
+
 
 =head1 BUGS
 
